@@ -72,7 +72,7 @@ def runs(*uuids: str):
 
     Arguments:
         uuids (str): UUIDs of the runs. You can
-            get this from `dashboard <https://github.com/lab-ml/lab_dashboard>`_
+            get this from `labml.ai app <https://github.com/labmlai/labml/tree/master/app>`_
 
     Example:
         >>> from labml import analytics
@@ -95,6 +95,12 @@ def get_run(uuid: str):
 
 
 def set_preferred_db(db: str):
+    """
+    Set the preference to load data from.
+
+    Arguments:
+        db (str): Either ``tensorboard`` or ``sqlite``
+    """
     if db not in ['tensorboard', 'sqlite']:
         raise ValueError('db should be one of tensorboard or sqlite')
     _cache.set_preferred_db(db)
@@ -195,7 +201,8 @@ def distribution(*args: any,
         step = args[1]
 
     if names is None:
-        names = [f'{i + 1}' for i in range(len(series))]
+        digits = len(str(len(series)))
+        names = [str(i + 1).zfill(digits) for i in range(len(series))]
 
     if series is None:
         raise ValueError("distribution should be called with an indicator collection"
