@@ -2,7 +2,7 @@ import json
 import os
 import pathlib
 import time
-from typing import Optional, List, Set, Dict, Union, TYPE_CHECKING
+from typing import Optional, List, Set, Dict, Union, TYPE_CHECKING, Any
 
 import git
 from labml.internal.api.dynamic import DynamicUpdateHandler
@@ -26,10 +26,31 @@ if TYPE_CHECKING:
 
 
 class ModelSaver:
+    """
+    An abstract class defining model saver/loader.
+
+    The implementation should keep a reference to the model and load and save the model
+    parameters.
+    """
     def save(self, checkpoint_path: pathlib.Path) -> any:
+        """
+        Saves the model in the given checkpoint path
+
+        Arguments:
+            checkpoint_path (pathlib.Path): The path to save the model at
+
+        Returns any meta info, such as the individual filename(s)
+        """
         raise NotImplementedError()
 
-    def load(self, checkpoint_path: pathlib.Path, info: any):
+    def load(self, checkpoint_path: pathlib.Path, info: any) -> None:
+        """
+        Loads the model from the given checkpoint path
+
+        Arguments:
+            checkpoint_path (pathlib.Path): The path to load the model from
+            info (any): The returned meta data when saving
+        """
         raise NotImplementedError()
 
 
@@ -157,6 +178,8 @@ class Experiment:
         tags (Set[str], optional): Set of tags for experiment
     """
     web_api: Optional['ApiExperiment']
+    wandb: Optional[Any]
+
     is_started: bool
     run: Run
     configs_processor: Optional[ConfigProcessor]
@@ -220,7 +243,10 @@ class Experiment:
         try:
             repo = git.Repo(lab_singleton().path)
 
-            self.run.repo_remotes = list(repo.remote().urls)
+            try:
+                self.run.repo_remotes = list(repo.remote().urls)
+            except (ValueError, git.GitCommandError):
+                self.run.repo_remotes = []
             self.run.commit = repo.head.commit.hexsha
             self.run.commit_message = repo.head.commit.message.strip()
             self.run.is_dirty = repo.is_dirty()
@@ -228,7 +254,7 @@ class Experiment:
         except git.InvalidGitRepositoryError:
             if not is_colab() and not is_kaggle():
                 labml_notice(["Not a valid git repository: ",
-                              (lab_singleton().path, Text.value)])
+                              (str(lab_singleton().path), Text.value)])
             self.run.commit = 'unknown'
             self.run.commit_message = ''
             self.run.is_dirty = True
@@ -237,6 +263,7 @@ class Experiment:
         self.checkpoint_saver = CheckpointSaver(self.run.checkpoint_path)
         self.is_evaluate = is_evaluate
         self.web_api = None
+        self.wandb = None
         self.writers = writers
         self.is_started = False
         self.distributed_rank = 0
@@ -331,7 +358,7 @@ class Experiment:
             self.run.pids_path.mkdir()
 
         pid_path = self.run.pids_path / f'{self.distributed_rank}.pid'
-        assert not pid_path.exists()
+        assert not pid_path.exists(), str(pid_path)
 
         with open(str(pid_path), 'w') as f:
             f.write(f'{os.getpid()}')
@@ -367,6 +394,13 @@ class Experiment:
             from labml.internal.tracker.writers import tensorboard
             tracker().add_writer(tensorboard.Writer(self.run.tensorboard_log_path))
 
+        if 'wandb' in self.writers:
+            from labml.internal.tracker.writers import wandb
+            self.wandb = wandb.Writer()
+            tracker().add_writer(self.wandb)
+        else:
+            self.wandb = None
+
         if 'file' in self.writers:
             from labml.internal.tracker.writers import file
             tracker().add_writer(file.Writer(self.run.log_file))
@@ -379,7 +413,7 @@ class Experiment:
                 from labml.internal.api.experiment import ApiExperiment
                 api_caller = ApiCaller(web_api_conf.url,
                                        {'run_uuid': self.run.uuid},
-                                       timeout_seconds=15)
+                                       timeout_seconds=120)
                 self.web_api = ApiExperiment(api_caller,
                                              frequency=web_api_conf.frequency,
                                              open_browser=web_api_conf.open_browser)
@@ -412,6 +446,9 @@ class Experiment:
 
         if not self.is_evaluate:
             if self.distributed_rank == 0:
+                from labml.internal.computer.configs import computer_singleton
+                computer_singleton().add_project(lab_singleton().path)
+
                 self.run.save_info()
             self._save_pid()
 
@@ -425,12 +462,12 @@ class Experiment:
                         self.configs_processor.add_saver(self.web_api.get_configs_saver())
                         self.web_api.set_dynamic_handler(ExperimentDynamicUpdateHandler(self.configs_processor))
 
-                tracker().save_indicators(self.run.indicators_path)
+                if self.wandb is not None:
+                    self.wandb.init(self.run.name, self.run.run_path)
+                    if self.configs_processor is not None:
+                        self.configs_processor.add_saver(self.wandb.get_configs_saver())
 
-                # PERF: Writing to tensorboard takes about 4 seconds
-                # Also wont work when configs are updated live
-                # if self.configs_processor:
-                #     tracker().write_h_parameters(self.configs_processor.get_hyperparams())
+                tracker().save_indicators(self.run.indicators_path)
 
         self.is_started = True
         return ExperimentWatcher(self)
