@@ -1,11 +1,14 @@
+import json
+import tarfile
 import time
 from pathlib import Path
-from typing import List, Dict, Optional
+from typing import List, Dict, Optional, Tuple
 
 import numpy as np
 
-from labml import logger
+from labml import logger, lab, monit
 from labml.internal.configs.processor import load_configs
+from labml.utils.download import download_file
 from .. import util
 from ..manage.runs import get_run_by_uuid, get_checkpoints
 from ...logger import Text
@@ -21,6 +24,11 @@ def struct_time_to_date(t: time.struct_time):
 
 
 _GLOBAL_STEP = 'global_step'
+_TAR_BUFFER_SIZE = 100_000
+
+
+class RunLoadError(TypeError):
+    pass
 
 
 class RunInfo:
@@ -76,6 +84,17 @@ class RunInfo:
         self.notes = notes
         self.tags = tags
 
+    @classmethod
+    def from_path(cls, run_path: Path):
+        """
+        ## Create a new from path
+        """
+        run_info_path = run_path / 'run.yaml'
+
+        with open(str(run_info_path), 'r') as f:
+            data = util.yaml_load(f.read())
+            return cls.from_dict(run_path.parent, data)
+
     @classmethod
     def from_dict(cls, experiment_path: Path, data: Dict[str, any]):
         """
@@ -83,7 +102,11 @@ class RunInfo:
         """
         params = dict(experiment_path=experiment_path)
         params.update(data)
-        return cls(**params)
+        try:
+            run = cls(**params)
+            return run
+        except TypeError as e:
+            raise RunLoadError(e)
 
     def to_dict(self):
         """
@@ -232,18 +255,16 @@ def _get_run_checkpoint(run_path: Path, checkpoint: int = -1):
     else:
         required_ci = checkpoint
 
-    checkpoint_steps = set(checkpoint_steps)
+    selected_ci = -1
+    for ci in checkpoint_steps:
+        if ci <= required_ci:
+            selected_ci = max(selected_ci, ci)
 
-    for ci in range(required_ci, -1, -1):
-        if ci not in checkpoint_steps:
-            continue
-
-        return ci
+    return selected_ci
 
 
-def get_run_checkpoint(run_uuid: str,
-                       checkpoint: int = -1):
-    run_path = get_run_by_uuid(run_uuid)
+def get_run_checkpoint(run_uuid: str, checkpoint: int = -1):
+    run_path = get_run_by_uuid(lab.get_experiments_path(), run_uuid)
     if run_path is None:
         logger.log("Couldn't find a previous run")
         return None, None
@@ -272,7 +293,7 @@ def get_run_checkpoint(run_uuid: str,
 
 
 def get_configs(run_uuid: str):
-    run_path = get_run_by_uuid(run_uuid)
+    run_path = get_run_by_uuid(lab.get_experiments_path(), run_uuid)
     if run_path is None:
         labml_notice(["Couldn't find a previous run to load configurations: ",
                       (run_uuid, Text.value)], is_danger=True)
@@ -282,3 +303,106 @@ def get_configs(run_uuid: str):
     configs = load_configs(configs_path)
 
     return configs
+
+
+def save_bundle(path: Path, run_uuid: str, checkpoint: int = -1, *,
+                data_files: List[str]):
+    run_path = get_run_by_uuid(lab.get_experiments_path(), run_uuid)
+    if run_path is None:
+        raise RuntimeError(f"Couldn't find run {run_uuid}")
+
+    checkpoint = _get_run_checkpoint(run_path, checkpoint)
+
+    if checkpoint is None:
+        raise RuntimeError(f"Couldn't find checkpoint {run_uuid}:{checkpoint}")
+
+    info_path = path.parent / f'{path.stem}.info.json'
+    info = {'uuid': run_uuid, 'checkpoint': checkpoint}
+    with open(str(info_path), 'w') as f:
+        f.write(json.dumps(info))
+
+    checkpoint_path = run_path / "checkpoints" / str(checkpoint)
+
+    with monit.section('Create bundle'):
+        with tarfile.open(str(path), 'w:gz') as tar:
+            tar.add(str(checkpoint_path), 'checkpoint')
+            tar.add(str(run_path / 'run.yaml'), 'run.yaml')
+            tar.add(str(run_path / 'configs.yaml'), 'configs.yaml')
+            tar.add(str(info_path), 'info.json')
+            for f in data_files:
+                tar.add(str(lab.get_data_path() / f), f'data/{f}')
+
+    info_path.unlink()
+
+
+def _extract_tar_file(tar: tarfile.TarFile, f: tarfile.TarInfo, path: Path):
+    with tar.extractfile(f) as ef:
+        with open(str(path), 'wb') as o:
+            while True:
+                r = ef.read(_TAR_BUFFER_SIZE)
+                if r:
+                    o.write(r)
+                else:
+                    break
+
+
+def load_bundle(path: Path, url: Optional[str] = None) -> Tuple[str, int]:
+    if url:
+        download_file(url, path)
+
+    if not path.exists():
+        raise FileNotFoundError(f'Bundle archive missing: {path}')
+
+    with monit.section('Extract bundle'):
+        with tarfile.open(str(path), 'r:gz') as tar:
+            files = tar.getmembers()
+            info_member = None
+            for f in files:
+                if f.name == 'info.json':
+                    info_member = f
+
+            if not info_member:
+                raise RuntimeError(f"Corrupted bundle. Missing info.json")
+
+            with tar.extractfile(info_member) as ef:
+                info = json.load(ef)
+
+            run_uuid, checkpoint = info['uuid'], info['checkpoint']
+            run_path = get_run_by_uuid(lab.get_experiments_path(), run_uuid)
+
+            if run_path is not None:
+                logger.log(f"Run {run_uuid} exists", Text.meta)
+                current_checkpoint = _get_run_checkpoint(run_path, checkpoint)
+                if checkpoint == current_checkpoint:
+                    logger.log(f"Checkpoint {checkpoint} exists", Text.meta)
+                    return run_uuid, checkpoint
+
+            run_path = lab.get_experiments_path() / 'bundled' / run_uuid
+
+            checkpoint_path = run_path / "checkpoints" / str(checkpoint)
+            if not checkpoint_path.exists():
+                checkpoint_path.mkdir(parents=True)
+
+            data_path = lab.get_data_path()
+            if not data_path.exists():
+                data_path.mkdir(parents=True)
+
+            for f in files:
+                if f.name == 'run.yaml':
+                    _extract_tar_file(tar, f, run_path / 'run.yaml')
+                elif f.name == 'configs.yaml':
+                    _extract_tar_file(tar, f, run_path / 'configs.yaml')
+                elif f.name.startswith('checkpoint/'):
+                    p = f.name[len('checkpoint/'):]
+                    p = checkpoint_path / p
+                    if not p.parent.exists():
+                        p.parent.mkdir(parents=True)
+                    _extract_tar_file(tar, f, p)
+                elif f.name.startswith('data/'):
+                    p = f.name[len('data/'):]
+                    p = data_path / p
+                    if not p.parent.exists():
+                        p.parent.mkdir(parents=True)
+                    _extract_tar_file(tar, f, p)
+
+            return run_uuid, checkpoint
