@@ -1,20 +1,15 @@
+import math
 from typing import TYPE_CHECKING, List, Tuple
 
-import numpy as np
-from ...logger import Text, StyleCode
+from labml.internal.util.colors import StyleCode
+from ...logger import Text
 
 if TYPE_CHECKING:
     from . import Logger
 
-try:
-    import torch
-except ImportError:
-    torch = None
 
-try:
-    import numpy
-except ImportError:
-    numpy = None
+def _format_bool(value: bool):
+    return 'true' if value else 'false'
 
 
 def _format_int(value: int):
@@ -25,7 +20,7 @@ def _format_float(value: float):
     if abs(value) < 1e-9:
         lg = 0
     else:
-        lg = int(np.ceil(np.log10(abs(value)))) + 1
+        lg = int(math.ceil(math.log10(abs(value)))) + 1
 
     decimals = 7 - lg
     decimals = max(1, decimals)
@@ -36,23 +31,41 @@ def _format_float(value: float):
 
 
 def _format_value(value: any):
-    if isinstance(value, int):
+    if isinstance(value, bool):
+        return _format_bool(value)
+    elif isinstance(value, int):
         return _format_int(value)
-    elif (isinstance(value, np.int) or
-          isinstance(value, np.long) or
-          isinstance(value, np.uint64)):
+    elif isinstance(value, float):
+        return _format_float(value)
+
+    try:
+        import numpy
+    except ImportError:
+        numpy = None
+
+    if numpy is None:
+        return None
+
+    if isinstance(value, numpy.number) and numpy.issubdtype(value, numpy.integer):
         return _format_int(int(value))
-    elif isinstance(value, np.float):
-        if np.isnan(value):
+    elif isinstance(value, numpy.number) and numpy.issubdtype(value, numpy.floating):
+        if numpy.isnan(value):
             return 'NaN'
         return _format_float(float(value))
-    elif torch is not None and isinstance(value, torch.Tensor):
-        if not value.shape:
-            return _format_value(value.item())
-        else:
-            return None
-    else:
-        return None
+
+    try:
+        import torch
+    except ImportError:
+        torch = None
+
+    if torch is not None:
+        if isinstance(value, torch.Tensor):
+            if not value.shape:
+                return _format_value(value.item())
+            else:
+                return None
+
+    return None
 
 
 def _format_tensor(s: List[str], limit: int = 1_000, style: StyleCode = Text.value):
@@ -67,9 +80,9 @@ def _format_tensor(s: List[str], limit: int = 1_000, style: StyleCode = Text.val
 
         if length > limit:
             res.append((' ... ', Text.warning))
-            break
+            return res, True
 
-    return res
+    return res, False
 
 
 def _key_value_pair(key: any, value: any, style: StyleCode = Text.meta):
@@ -80,27 +93,39 @@ def _key_value_pair(key: any, value: any, style: StyleCode = Text.meta):
             (f, style)]
 
 
-def _render_tensor(tensor, *, new_line: str = '\n', indent: str = ''):
+def _render_tensor(tensor, *, new_line: str = '\n', indent: str = '', depth=0):
     if len(tensor) > 5:
+        truncated = True
         idx = [0, 1, 2, '...', len(tensor) - 1]
     else:
+        truncated = False
         idx = [i for i in range(len(tensor))]
 
     res = [indent, '[']
-    if new_line == '\n':
-        next_indent = ' ' + indent
+    if depth >= 2:
+        new_line = ''
     else:
-        next_indent = indent
+        new_line = new_line
+
+    if new_line == '\n':
+        current_indent = ' ' + indent
+    else:
+        indent = ''
+        current_indent = ''
     if len(tensor.shape) > 1:
         res.append(new_line)
         for i in idx:
             if i == '...':
-                res.append(next_indent)
+                res.append(current_indent)
                 res.append('...')
             else:
-                res += _render_tensor(tensor[i],
-                                      new_line=new_line,
-                                      indent=next_indent)
+                sub_res, sub_trunc = _render_tensor(tensor[i],
+                                                    new_line=new_line,
+                                                    indent=current_indent,
+                                                    depth=depth + 1,
+                                                    )
+                truncated = truncated or sub_trunc
+                res += sub_res
             if i != idx[-1]:
                 res.append(', ')
             res.append(new_line)
@@ -109,14 +134,14 @@ def _render_tensor(tensor, *, new_line: str = '\n', indent: str = ''):
             if i == '...':
                 res.append('...')
             else:
-                res += _format_value(tensor[i])
+                res += [_format_value(tensor[i])]
             if i != idx[-1]:
                 res.append(', ')
 
     res.append(indent)
     res.append(']')
 
-    return res
+    return res, truncated
 
 
 def _get_value_full(value: any):
@@ -134,34 +159,55 @@ def _get_value_full(value: any):
                     (_format_int(len(value)), Text.meta),
                     (')', Text.subtle)]
 
-    elif numpy is not None and isinstance(value, numpy.ndarray):
-        return [*_key_value_pair('dtype', value.dtype),
-                '\n',
-                *_key_value_pair('shape', [s for s in value.shape], Text.value),
-                '\n',
-                *_key_value_pair('min', numpy.min(value)),
-                ' ',
-                *_key_value_pair('max', numpy.max(value)),
-                ' ',
-                *_key_value_pair('mean', numpy.mean(value)),
-                ' ',
-                *_key_value_pair('std', numpy.std(value)),
-                '\n',
-                *_format_tensor(_render_tensor(value, new_line='\n'))]
-    elif torch is not None and isinstance(value, torch.Tensor):
-        return [*_key_value_pair('dtype', value.dtype),
-                '\n',
-                *_key_value_pair('shape', [s for s in value.shape], Text.value),
-                '\n',
-                *_key_value_pair('min', torch.min(value).item()),
-                ' ',
-                *_key_value_pair('max', torch.max(value).item()),
-                ' ',
-                *_key_value_pair('mean', torch.mean(value.to(torch.float)).item()),
-                ' ',
-                *_key_value_pair('std', torch.std(value.to(torch.float)).item()),
-                '\n',
-                *_format_tensor(_render_tensor(value, new_line='\n'))]
+    try:
+        import numpy
+    except ImportError:
+        numpy = None
+
+    if numpy is not None and isinstance(value, numpy.ndarray):
+        arr, trunc = _render_tensor(value, new_line='\n')
+        arr, trunc_format = _format_tensor(arr)
+        if not trunc and not trunc_format:
+            return arr
+        else:
+            return [*_key_value_pair('dtype', value.dtype),
+                    '\n',
+                    *_key_value_pair('shape', [s for s in value.shape], Text.value),
+                    '\n',
+                    *_key_value_pair('min', numpy.min(value)),
+                    ' ',
+                    *_key_value_pair('max', numpy.max(value)),
+                    ' ',
+                    *_key_value_pair('mean', numpy.mean(value)),
+                    ' ',
+                    *_key_value_pair('std', numpy.std(value)),
+                    '\n',
+                    *arr]
+
+    try:
+        import torch
+    except ImportError:
+        torch = None
+
+    if torch is not None and isinstance(value, torch.Tensor):
+        arr, trunc = _render_tensor(value, new_line='\n')
+        arr, trunc_format = _format_tensor(arr)
+        if not trunc and not trunc_format:
+            return arr
+        else:
+            return [*_key_value_pair('dtype', value.dtype),
+                    '\n',
+                    *_key_value_pair('shape', [s for s in value.shape], Text.value),
+                    '\n',
+                    *_key_value_pair('min', torch.min(value).item()),
+                    ' ',
+                    *_key_value_pair('max', torch.max(value).item()),
+                    ' ',
+                    *_key_value_pair('mean', torch.mean(value.to(torch.float)).item()),
+                    ' ',
+                    *_key_value_pair('std', torch.std(value.to(torch.float)).item()),
+                    '\n',
+                    *arr]
 
     s = str(value)
     s = s.replace('\r', '')
@@ -195,10 +241,22 @@ def _get_value_line(value: any):
         return [(f, Text.value)]
     elif isinstance(value, str):
         return [('"', Text.subtle)] + _shrink(value) + [('"', Text.subtle)]
-    elif numpy is not None and isinstance(value, numpy.ndarray):
-        return [*_format_tensor(_render_tensor(value, new_line=''), limit=80)]
-    elif torch is not None and isinstance(value, torch.Tensor):
-        return [*_format_tensor(_render_tensor(value, new_line=''), limit=80)]
+
+    try:
+        import numpy
+    except ImportError:
+        numpy = None
+
+    if numpy is not None and isinstance(value, numpy.ndarray):
+        return [*_format_tensor(_render_tensor(value, new_line='')[0], limit=80)[0]]
+
+    try:
+        import torch
+    except ImportError:
+        torch = None
+
+    if torch is not None and isinstance(value, torch.Tensor):
+        return [*_format_tensor(_render_tensor(value, new_line='')[0], limit=80)[0]]
 
     s = str(value)
     return _shrink(s)
@@ -235,9 +293,9 @@ class Inspect:
         elif len(args) == 1:
             assert len(kwargs.keys()) == 0
             arg = args[0]
-            if type(arg) == list:
+            if isinstance(arg, list):
                 self._log_key_value([(i, v) for i, v in enumerate(arg)])
-            elif type(arg) == dict:
+            elif isinstance(arg, dict):
                 keys = list(arg.keys())
                 keys.sort()
                 self._log_key_value([(k, arg[k]) for k in keys])
