@@ -1,12 +1,8 @@
 from typing import Dict
 
 import numpy as np
-from .numeric import Queue, Histogram, Scalar
 
-try:
-    import torch
-except ImportError:
-    torch = None
+from .numeric import Queue, Histogram, Scalar
 
 
 def load_indicator_from_dict(data: Dict[str, any]):
@@ -32,7 +28,14 @@ def create_default_indicator(name: str, value: any, is_print: bool):
         return Scalar(name, is_print)
     elif isinstance(value, np.ndarray):
         return Scalar(name, is_print)
-    elif torch is not None:
+
+    try:
+        import torch
+    except ImportError:
+        torch = None
+
+    if torch is not None:
+        from torch.optim.optimizer import Optimizer
         if isinstance(value, torch.nn.parameter.Parameter):
             return Scalar(name, is_print)
         elif isinstance(value, torch.Tensor):
@@ -40,5 +43,11 @@ def create_default_indicator(name: str, value: any, is_print: bool):
         elif isinstance(value, torch.nn.Module):
             from labml.internal.tracker.indicators.aggregate import PyTorchModule
             return PyTorchModule(name, is_print)
+        elif isinstance(value, Optimizer):
+            from labml.internal.tracker.indicators.aggregate import PyTorchOptimizer
+            return PyTorchOptimizer(name, is_print)
+        elif isinstance(value, tuple) and isinstance(value[0], Optimizer):
+            from labml.internal.tracker.indicators.aggregate import PyTorchOptimizer
+            return PyTorchOptimizer(name, is_print)
 
     raise ValueError(f"Unknown type {type(value)}")
